@@ -102,7 +102,7 @@ class PostAdmin(admin.ModelAdmin):
 
 
 class ProfileAdmin(admin.ModelAdmin):
-    list_display = ['user', 'time_zone', 'language']
+    list_display = ['user', 'time_zone', 'language', 'post_count']
     list_per_page = 20
     ordering = ['-user']
     search_fields = ['user__username', 'user__first_name', 'user__last_name']
