@@ -43,7 +43,7 @@ class PollAnswerForm(forms.ModelForm):
 class BasePollAnswerFormset(BaseInlineFormSet):
     def clean(self):
         if any(self.errors):
-            return
+            raise forms.ValidationError(self.errors)
         forms_cnt = len(self.initial_forms) + len([form for form in self.extra_forms if form.has_changed()]) -\
                     len(self.deleted_forms)
         if forms_cnt > defaults.PYBB_POLL_MAX_ANSWERS:
@@ -124,9 +124,12 @@ class PostForm(forms.ModelForm):
                 post.topic.poll_type = self.cleaned_data['poll_type']
                 post.topic.poll_question = self.cleaned_data['poll_question']
                 post.topic.updated = tznow()
-                post.topic.save()
-            post.save()
+                if commit:
+                    post.topic.save()
+            if commit:
+                post.save()
             return post
+
         allow_post = True
         if defaults.PYBB_PREMODERATION:
             allow_post = defaults.PYBB_PREMODERATION(self.user, self.cleaned_data['body'])
@@ -140,14 +143,16 @@ class PostForm(forms.ModelForm):
             )
             if not allow_post:
                 topic.on_moderation = True
-            topic.save()
+            if commit:
+                topic.save()
         else:
             topic = self.topic
         post = Post(topic=topic, user=self.user, user_ip=self.ip,
             body=self.cleaned_data['body'])
         if not allow_post:
             post.on_moderation = True
-        post.save()
+        if commit:
+            post.save()
         return post
 
 
