@@ -3,9 +3,10 @@
 import os.path
 import uuid
 
-from django.db import models
+from django.db import models, transaction
 from django.contrib.auth.models import User
 from django.core.urlresolvers import reverse
+from django.db.utils import IntegrityError
 from django.utils.html import strip_tags
 from django.utils.translation import ugettext_lazy as _
 from django.conf import settings
@@ -110,11 +111,15 @@ class Forum(models.Model):
         return self.name
 
     def update_counters(self):
-        self.post_count = Post.objects.filter(topic__forum=self).count()
+        posts = Post.objects.filter(topic__forum__id=self.id)
+        self.post_count = posts.count()
         self.topic_count = Topic.objects.filter(forum=self).count()
-        last_post = self.get_last_post()
-        if last_post:
-            self.updated = self.last_post.updated or self.last_post.created
+        try:
+            last_post = posts.order_by('-created')[0]
+            self.updated = last_post.updated or last_post.created
+        except IndexError:
+            pass
+
         self.save()
 
     def get_absolute_url(self):
@@ -126,7 +131,7 @@ class Forum(models.Model):
 
     def get_last_post(self):
         try:
-            return self.posts.order_by('-created').select_related()[0]
+            return self.posts.order_by('-created')[0]
         except IndexError:
             return None
 
@@ -188,7 +193,7 @@ class Topic(models.Model):
         return self._head[0]
 
     def get_last_post(self):
-        return self.posts.order_by('-created').select_related()[0]
+        return self.posts.order_by('-created').select_related('user')[0]
 
     @property
     def last_post(self):
@@ -200,11 +205,28 @@ class Topic(models.Model):
     def save(self, *args, **kwargs):
         if self.id is None:
             self.created = tznow()
+
+        forum_changed = False
+        old_topic = None
+        if self.id is not None:
+            old_topic = Topic.objects.get(id=self.id)
+            if self.forum != old_topic.forum:
+                forum_changed = True
+
         super(Topic, self).save(*args, **kwargs)
 
+        if forum_changed:
+            old_topic.forum.update_counters()
+            self.forum.update_counters()
+
+    def delete(self, using=None):
+        super(Topic, self).delete(using)
+        self.forum.update_counters()
+
     def update_counters(self):
         self.post_count = self.posts.count()
-        self.updated = self.last_post.updated or self.last_post.created
+        last_post = Post.objects.filter(topic__id=self.id).order_by('-created')[0]
+        self.updated = last_post.updated or last_post.created
         self.save()
 
     def get_parents(self):
@@ -243,7 +265,7 @@ class RenderableItem(models.Model):
 class Post(RenderableItem):
     topic = models.ForeignKey(Topic, related_name='posts', verbose_name=_('Topic'))
     user = models.ForeignKey(User, related_name='posts', verbose_name=_('User'))
-    created = models.DateTimeField(_('Created'), blank=True)
+    created = models.DateTimeField(_('Created'), blank=True, db_index=True)
     updated = models.DateTimeField(_('Updated'), blank=True, null=True)
     user_ip = models.IPAddressField(_('User IP'), blank=True, default='0.0.0.0')
     on_moderation = models.BooleanField(_('On moderation'), default=False)
@@ -268,17 +290,26 @@ class Post(RenderableItem):
 
         new = self.pk is None
 
+        topic_changed = False
+        old_post = None
+        if not new:
+            old_post = Post.objects.get(pk=self.pk)
+            if old_post.topic != self.topic:
+                topic_changed = True
+
         super(Post, self).save(*args, **kwargs)
 
-        if new:
-            self.topic.updated = created_at
-            self.topic.forum.updated = created_at
         # If post is topic head and moderated, moderate topic too
-        if self.topic.head == self and self.on_moderation == False and self.topic.on_moderation == True:
+        if self.topic.head == self and not self.on_moderation and self.topic.on_moderation:
             self.topic.on_moderation = False
+
         self.topic.update_counters()
         self.topic.forum.update_counters()
 
+        if topic_changed:
+            old_post.topic.update_counters()
+            old_post.topic.forum.update_counters()
+
     def get_absolute_url(self):
         return reverse('pybb:post', kwargs={'pk': self.id})
 
@@ -291,8 +322,7 @@ class Post(RenderableItem):
         else:
             super(Post, self).delete(*args, **kwargs)
             self.topic.update_counters()
-
-        self.topic.forum.update_counters()
+            self.topic.forum.update_counters()
 
     def get_parents(self):
         """
@@ -382,31 +412,77 @@ class Attachment(models.Model):
             return '%.2fMb' % (size / float(1024 * 1024))
 
 
+class TopicReadTrackerManager(models.Manager):
+    @transaction.commit_on_success
+    def get_or_create_tracker(self, user, topic):
+        """
+        Correctly create tracker in mysql db on default REPEATABLE READ transaction mode
+
+        It's known problem when standrard get_or_create method return can raise exception
+        with correct data in mysql database.
+        See http://stackoverflow.com/questions/2235318/how-do-i-deal-with-this-race-condition-in-django/2235624
+        """
+        is_new = True
+        try:
+            obj = TopicReadTracker.objects.create(user=user, topic=topic)
+        except IntegrityError:
+            transaction.commit()
+            obj = TopicReadTracker.objects.get(user=user, topic=topic)
+            is_new = False
+        return obj, is_new
+
+
 class TopicReadTracker(models.Model):
     """
     Save per user topic read tracking
     """
-    class Meta(object):
-        verbose_name = _('Topic read tracker')
-        verbose_name_plural = _('Topic read trackers')
-
     user = models.ForeignKey(User, blank=False, null=False)
     topic = models.ForeignKey(Topic, blank=True, null=True)
     time_stamp = models.DateTimeField(auto_now=True)
 
+    objects = TopicReadTrackerManager()
+
+    class Meta(object):
+        verbose_name = _('Topic read tracker')
+        verbose_name_plural = _('Topic read trackers')
+        unique_together = ('user', 'topic')
+
+
+class ForumReadTrackerManager(models.Manager):
+    @transaction.commit_on_success
+    def get_or_create_tracker(self, user, forum):
+        """
+        Correctly create tracker in mysql db on default REPEATABLE READ transaction mode
+
+        It's known problem when standrard get_or_create method return can raise exception
+        with correct data in mysql database.
+        See http://stackoverflow.com/questions/2235318/how-do-i-deal-with-this-race-condition-in-django/2235624
+        """
+        is_new = True
+        try:
+            obj = ForumReadTracker.objects.create(user=user, forum=forum)
+        except IntegrityError:
+            transaction.commit()
+            is_new = False
+            obj = ForumReadTracker.objects.get(user=user, forum=forum)
+        return obj, is_new
+
 
 class ForumReadTracker(models.Model):
     """
     Save per user forum read tracking
     """
-    class Meta(object):
-        verbose_name = _('Forum read tracker')
-        verbose_name_plural = _('Forum read trackers')
-
     user = models.ForeignKey(User, blank=False, null=False)
     forum = models.ForeignKey(Forum, blank=True, null=True)
     time_stamp = models.DateTimeField(auto_now=True)
 
+    objects = ForumReadTrackerManager()
+
+    class Meta(object):
+        verbose_name = _('Forum read tracker')
+        verbose_name_plural = _('Forum read trackers')
+        unique_together = ('user', 'forum')
+
 
 class PollAnswer(models.Model):
     topic = models.ForeignKey(Topic, related_name='poll_answers', verbose_name=_('Topic'))
