@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 
 import math
+from string import strip
 import time
 
 from django import template
@@ -25,6 +26,7 @@ except ImportError:
     pytils_enabled = False
 
 from pybb.models import TopicReadTracker, ForumReadTracker, PollAnswerUser
+from pybb.permissions import perms
 from pybb import defaults
 
 
@@ -105,21 +107,14 @@ def pybb_topic_moderated_by(topic, user):
     Check if user is moderator of topic's forum.
     """
 
-    return user.is_superuser or (user in topic.forum.moderators.all())
+    return perms.may_moderate_topic(user, topic)
 
 @register.filter
 def pybb_editable_by(post, user):
     """
     Check if the post could be edited by the user.
     """
-
-    if user.is_superuser:
-        return True
-    if post.user == user:
-        return True
-    if user in post.topic.forum.moderators.all():
-        return True
-    return False
+    return perms.may_edit_post(user, post)    
 
 
 @register.filter
@@ -129,28 +124,47 @@ def pybb_posted_by(post, user):
     """
     return post.user == user
 
+
+@register.filter
+def pybb_is_topic_unread(topic, user):
+    if not user.is_authenticated():
+        return False
+
+    last_topic_update = topic.updated or topic.created
+
+    unread = not ForumReadTracker.objects.filter(
+        forum=topic.forum,
+        user=user.id,
+        time_stamp__gte=last_topic_update).exists()
+    unread &= not TopicReadTracker.objects.filter(
+        topic=topic,
+        user=user.id,
+        time_stamp__gte=last_topic_update).exists()
+    return unread
+
+
 @register.filter
 def pybb_topic_unread(topics, user):
     """
     Mark all topics in queryset/list with .unread for target user
     """
     topic_list = list(topics)
+
     if user.is_authenticated():
         for topic in topic_list:
             topic.unread = True
-        try:
-            forum_mark = ForumReadTracker.objects.get(user=user, forum=topic_list[0].forum)
-        except:
-            forum_mark = None
-        qs = TopicReadTracker.objects.filter(
-                user=user,
-                topic__in=topic_list
-                ).select_related('topic')
-        if forum_mark:
-            qs = qs.filter(topic__updated__gt=forum_mark.time_stamp)
+
+        forums_ids = [f.forum_id for f in topic_list]
+        forum_marks = dict([(m.forum_id, m.time_stamp)
+                            for m
+                            in ForumReadTracker.objects.filter(user=user, forum__in=forums_ids)])
+        if len(forum_marks):
             for topic in topic_list:
-                if topic.updated and (topic.updated <= forum_mark.time_stamp):
+                topic_updated = topic.updated or topic.created
+                if topic.forum.id in forum_marks and topic_updated <= forum_marks[topic.forum.id]:
                     topic.unread = False
+
+        qs = TopicReadTracker.objects.filter(user=user, topic__in=topic_list).select_related('topic')
         topic_marks = list(qs)
         topic_dict = dict(((topic.id, topic) for topic in topic_list))
         for mark in topic_marks:
@@ -167,8 +181,7 @@ def pybb_forum_unread(forums, user):
     forum_list = list(forums)
     if user.is_authenticated():
         for forum in forum_list:
-            if forum.topic_count:
-                forum.unread = True
+            forum.unread = forum.topic_count > 0
         forum_marks = ForumReadTracker.objects.filter(
                 user=user,
                 forum__in=forum_list
@@ -180,6 +193,7 @@ def pybb_forum_unread(forums, user):
                 forum_dict[mark.forum.id].unread = False
     return forum_list
 
+
 @register.filter
 def pybb_topic_inline_pagination(topic):
     page_count = int(math.ceil(topic.post_count / float(defaults.PYBB_TOPIC_PAGE_SIZE)))
@@ -187,6 +201,12 @@ def pybb_topic_inline_pagination(topic):
         return range(1, page_count+1)
     return range(1, 5) + ['...', page_count]
 
+
 @register.filter
 def pybb_topic_poll_not_voted(topic, user):
-    return not PollAnswerUser.objects.filter(poll_answer__topic=topic, user=user).exists()
\ No newline at end of file
+    return not PollAnswerUser.objects.filter(poll_answer__topic=topic, user=user).exists()
+
+
+@register.filter
+def endswith(str, substr):
+    return str.endswith(substr)
\ No newline at end of file
