@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -*-
 
-import time
+import time, datetime
 import os
 
 from django.contrib.auth.models import User, Permission
 from django.core import mail
 from django.core.urlresolvers import reverse
 from django.core.exceptions import ValidationError
-from django.test import TransactionTestCase
+from django.test import TestCase
 from django.test.client import Client
+from pybb.templatetags.pybb_tags import pybb_is_topic_unread, pybb_topic_unread, pybb_forum_unread
 
 try:
     from lxml import html
@@ -43,8 +44,15 @@ class SharedTestModule(object):
     def get_form_values(self, response, form="post-form"):
         return dict(html.fromstring(response.content).xpath('//form[@class="%s"]' % form)[0].form_values())
 
+    def get_with_user(self, url, username=None, password=None):
+        if username: 
+            self.client.login(username=username, password=password)
+        r = self.client.get(url)
+        self.client.logout()
+        return r
 
-class FeaturesTest(TransactionTestCase, SharedTestModule):
+
+class FeaturesTest(TestCase, SharedTestModule):
     def setUp(self):
         self.ORIG_PYBB_ENABLE_ANONYMOUS_POST = defaults.PYBB_ENABLE_ANONYMOUS_POST
         self.ORIG_PYBB_PREMODERATION = defaults.PYBB_PREMODERATION
@@ -246,6 +254,10 @@ class FeaturesTest(TransactionTestCase, SharedTestModule):
         self.assertEqual(ForumReadTracker.objects.all().count(), 1)
         self.assertEqual(ForumReadTracker.objects.filter(user=user_bob).count(), 1)
         self.assertEqual(ForumReadTracker.objects.filter(user=user_bob, forum=self.forum).count(), 1)
+        self.assertEqual(TopicReadTracker.objects.filter(user=user_bob).count(), 0)
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2], user_bob)],
+            [False, False])
 
         # user_ann creates topic_3, they should get a new topic read tracker in the db 
         add_topic_url = reverse('pybb:add_topic', kwargs={'forum_id': self.forum.id})
@@ -334,6 +346,191 @@ class FeaturesTest(TransactionTestCase, SharedTestModule):
         self.assertEqual(ForumReadTracker.objects.filter(user=self.user).count(), 1)
         self.assertEqual(ForumReadTracker.objects.filter(user=self.user, forum=self.forum).count(), 1)
 
+    def test_pybb_is_topic_unread_filter(self):
+        forum_1 = self.forum
+        topic_1 = self.topic
+        topic_2 = Topic.objects.create(name='topic_2', forum=forum_1, user=self.user)
+
+        forum_2 = Forum.objects.create(name='forum_2', description='forum2', category=self.category)
+        topic_3 = Topic.objects.create(name='topic_2', forum=forum_2, user=self.user)
+
+        Post(topic=topic_1, user=self.user, body='one').save()
+        Post(topic=topic_2, user=self.user, body='two').save()
+        Post(topic=topic_3, user=self.user, body='three').save()
+
+        user_ann = User.objects.create_user('ann', 'ann@localhost', 'ann')
+        client_ann = Client()
+        client_ann.login(username='ann', password='ann')
+
+        # Two topics, each with one post. everything is unread, so the db should reflect that:
+        self.assertTrue(pybb_is_topic_unread(topic_1, user_ann))
+        self.assertTrue(pybb_is_topic_unread(topic_2, user_ann))
+        self.assertTrue(pybb_is_topic_unread(topic_3, user_ann))
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2, topic_3], user_ann)],
+            [True, True, True])
+
+        client_ann.get(topic_1.get_absolute_url())
+        topic_1 = Topic.objects.get(id=topic_1.id)
+        topic_2 = Topic.objects.get(id=topic_2.id)
+        topic_3 = Topic.objects.get(id=topic_3.id)
+        self.assertFalse(pybb_is_topic_unread(topic_1, user_ann))
+        self.assertTrue(pybb_is_topic_unread(topic_2, user_ann))
+        self.assertTrue(pybb_is_topic_unread(topic_3, user_ann))
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2, topic_3], user_ann)],
+            [False, True, True])
+
+        client_ann.get(topic_2.get_absolute_url())
+        topic_1 = Topic.objects.get(id=topic_1.id)
+        topic_2 = Topic.objects.get(id=topic_2.id)
+        topic_3 = Topic.objects.get(id=topic_3.id)
+        self.assertFalse(pybb_is_topic_unread(topic_1, user_ann))
+        self.assertFalse(pybb_is_topic_unread(topic_2, user_ann))
+        self.assertTrue(pybb_is_topic_unread(topic_3, user_ann))
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2, topic_3], user_ann)],
+            [False, False, True])
+
+        client_ann.get(topic_3.get_absolute_url())
+        topic_1 = Topic.objects.get(id=topic_1.id)
+        topic_2 = Topic.objects.get(id=topic_2.id)
+        topic_3 = Topic.objects.get(id=topic_3.id)
+        self.assertFalse(pybb_is_topic_unread(topic_1, user_ann))
+        self.assertFalse(pybb_is_topic_unread(topic_2, user_ann))
+        self.assertFalse(pybb_is_topic_unread(topic_3, user_ann))
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2, topic_3], user_ann)],
+            [False, False, False])
+
+    def test_read_tracker_when_topics_forum_changed(self):
+        forum_1 = Forum.objects.create(name='f1', description='bar', category=self.category)
+        forum_2 = Forum.objects.create(name='f2', description='bar', category=self.category)
+        topic_1 = Topic.objects.create(name='t1', forum=forum_1, user=self.user)
+        topic_2 = Topic.objects.create(name='t2', forum=forum_2, user=self.user)
+
+        Post.objects.create(topic=topic_1, user=self.user, body='one')
+        Post.objects.create(topic=topic_2, user=self.user, body='two')
+
+        user_ann = User.objects.create_user('ann', 'ann@localhost', 'ann')
+        client_ann = Client()
+        client_ann.login(username='ann', password='ann')
+
+        # Everything is unread
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2], user_ann)],
+            [True, True])
+        self.assertListEqual(
+            [t.unread for t in pybb_forum_unread([forum_1, forum_2], user_ann)],
+            [True, True])
+
+        # read all
+        client_ann.get(reverse('pybb:mark_all_as_read'))
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2], user_ann)],
+            [False, False])
+        self.assertListEqual(
+            [t.unread for t in pybb_forum_unread([forum_1, forum_2], user_ann)],
+            [False, False])
+
+        post = Post.objects.create(topic=topic_1, user=self.user, body='three')
+
+        topic_1 = Topic.objects.get(id=topic_1.id)
+        topic_2 = Topic.objects.get(id=topic_2.id)
+        self.assertEqual(topic_1.updated, post.updated or post.created)
+        self.assertEqual(forum_1.updated, post.updated or post.created)
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2], user_ann)],
+            [True, False])
+        self.assertListEqual(
+            [t.unread for t in pybb_forum_unread([forum_1, forum_2], user_ann)],
+            [True, False])
+
+        post.topic = topic_2
+        post.save()
+        topic_1 = Topic.objects.get(id=topic_1.id)
+        topic_2 = Topic.objects.get(id=topic_2.id)
+        forum_1 = Forum.objects.get(id=forum_1.id)
+        forum_2 = Forum.objects.get(id=forum_2.id)
+        self.assertEqual(topic_2.updated, post.updated or post.created)
+        self.assertEqual(forum_2.updated, post.updated or post.created)
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2], user_ann)],
+            [False, True])
+        self.assertListEqual(
+            [t.unread for t in pybb_forum_unread([forum_1, forum_2], user_ann)],
+            [False, True])
+
+        topic_2.forum = forum_1
+        topic_2.save()
+        topic_1 = Topic.objects.get(id=topic_1.id)
+        topic_2 = Topic.objects.get(id=topic_2.id)
+        forum_1 = Forum.objects.get(id=forum_1.id)
+        forum_2 = Forum.objects.get(id=forum_2.id)
+        self.assertEqual(forum_1.updated, post.updated or post.created)
+        self.assertListEqual(
+            [t.unread for t in pybb_topic_unread([topic_1, topic_2], user_ann)],
+            [False, True])
+        self.assertListEqual(
+            [t.unread for t in pybb_forum_unread([forum_1, forum_2], user_ann)],
+            [True, False])
+
+    def test_latest_topics(self):
+        topic_1 = self.topic
+        topic_1.updated = datetime.datetime.utcnow()
+        topic_2 = Topic.objects.create(name='topic_2', forum=self.forum, user=self.user)
+        topic_2.updated = datetime.datetime.utcnow() + datetime.timedelta(days=-1)
+
+        category_2 = Category.objects.create(name='cat2')
+        forum_2 = Forum.objects.create(name='forum_2', category=category_2)
+        topic_3 = Topic.objects.create(name='topic_3', forum=forum_2, user=self.user)
+        topic_3.updated = datetime.datetime.utcnow() + datetime.timedelta(days=-2)
+
+        self.login_client()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertEqual(response.status_code, 200)
+        self.assertListEqual(list(response.context['topic_list']), [topic_1, topic_2, topic_3])
+
+        topic_2.forum.hidden = True
+        topic_2.forum.save()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_3])
+
+        topic_2.forum.hidden = False
+        topic_2.forum.save()
+        category_2.hidden = True
+        category_2.save()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_1, topic_2])
+
+        topic_2.forum.hidden = False
+        topic_2.forum.save()
+        category_2.hidden = False
+        category_2.save()
+        topic_1.on_moderation = True
+        topic_1.save()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_1, topic_2, topic_3])
+
+        topic_1.user = User.objects.create_user('another', 'another@localhost', 'another')
+        topic_1.save()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_2, topic_3])
+
+        topic_1.forum.moderators.add(self.user)
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_1, topic_2, topic_3])
+
+        topic_1.forum.moderators.remove(self.user)
+        self.user.is_superuser = True
+        self.user.save()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_1, topic_2, topic_3])
+
+        self.client.logout()
+        response = self.client.get(reverse('pybb:topic_latest'))
+        self.assertListEqual(list(response.context['topic_list']), [topic_2, topic_3])
+
     def test_hidden(self):
         client = Client()
         category = Category(name='hcat', hidden=True)
@@ -356,9 +553,9 @@ class FeaturesTest(TransactionTestCase, SharedTestModule):
 
         
         self.assertFalse(category.id in [c.id for c in client.get(reverse('pybb:index')).context['categories']])
-        self.assertEqual(client.get(category.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(forum_in_hidden.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(topic_in_hidden.get_absolute_url()).status_code, 404)
+        self.assertEqual(client.get(category.get_absolute_url()).status_code, 302)
+        self.assertEqual(client.get(forum_in_hidden.get_absolute_url()).status_code, 302)
+        self.assertEqual(client.get(topic_in_hidden.get_absolute_url()).status_code, 302)
 
         self.assertNotContains(client.get(reverse('pybb:index')), forum_hidden.get_absolute_url())
         self.assertNotContains(client.get(reverse('pybb:feed_topics')), topic_hidden.get_absolute_url())
@@ -366,26 +563,26 @@ class FeaturesTest(TransactionTestCase, SharedTestModule):
 
         self.assertNotContains(client.get(reverse('pybb:feed_posts')), post_hidden.get_absolute_url())
         self.assertNotContains(client.get(reverse('pybb:feed_posts')), post_in_hidden.get_absolute_url())
-        self.assertEqual(client.get(forum_hidden.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(topic_hidden.get_absolute_url()).status_code, 404)
+        self.assertEqual(client.get(forum_hidden.get_absolute_url()).status_code, 302)
+        self.assertEqual(client.get(topic_hidden.get_absolute_url()).status_code, 302)
 
         client.login(username='zeus', password='zeus')
         self.assertFalse(category.id in [c.id for c in client.get(reverse('pybb:index')).context['categories']])
         self.assertNotContains(client.get(reverse('pybb:index')), forum_hidden.get_absolute_url())
-        self.assertEqual(client.get(category.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(forum_in_hidden.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(topic_in_hidden.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(forum_hidden.get_absolute_url()).status_code, 404)
-        self.assertEqual(client.get(topic_hidden.get_absolute_url()).status_code, 404)
+        self.assertEqual(client.get(category.get_absolute_url()).status_code, 403)
+        self.assertEqual(client.get(forum_in_hidden.get_absolute_url()).status_code, 403)
+        self.assertEqual(client.get(topic_in_hidden.get_absolute_url()).status_code, 403)
+        self.assertEqual(client.get(forum_hidden.get_absolute_url()).status_code, 403)
+        self.assertEqual(client.get(topic_hidden.get_absolute_url()).status_code, 403)
         self.user.is_staff = True
         self.user.save()
         self.assertTrue(category.id in [c.id for c in client.get(reverse('pybb:index')).context['categories']])
         self.assertContains(client.get(reverse('pybb:index')), forum_hidden.get_absolute_url())
-        self.assertNotEqual(client.get(category.get_absolute_url()).status_code, 404)
-        self.assertNotEqual(client.get(forum_in_hidden.get_absolute_url()).status_code, 404)
-        self.assertNotEqual(client.get(topic_in_hidden.get_absolute_url()).status_code, 404)
-        self.assertNotEqual(client.get(forum_hidden.get_absolute_url()).status_code, 404)
-        self.assertNotEqual(client.get(topic_hidden.get_absolute_url()).status_code, 404)
+        self.assertEqual(client.get(category.get_absolute_url()).status_code, 200)
+        self.assertEqual(client.get(forum_in_hidden.get_absolute_url()).status_code, 200)
+        self.assertEqual(client.get(topic_in_hidden.get_absolute_url()).status_code, 200)
+        self.assertEqual(client.get(forum_hidden.get_absolute_url()).status_code, 200)
+        self.assertEqual(client.get(topic_hidden.get_absolute_url()).status_code, 200)
 
     def test_inactive(self):
         self.login_client()
@@ -529,8 +726,23 @@ class FeaturesTest(TransactionTestCase, SharedTestModule):
         response = client.get(reverse('pybb:add_subscription', args=[self.topic.id]), follow=True)
         self.assertEqual(response.status_code, 200)
         self.assertTrue(user in list(self.topic.subscribers.all()))
-        new_post = Post(topic=self.topic, user=self.user, body='test subscribtion юникод')
-        new_post.save()
+
+        # create a new reply (with another user)
+        self.client.login(username='zeus', password='zeus')
+        add_post_url = reverse('pybb:add_post', args=[self.topic.id])
+        response = self.client.get(add_post_url)
+        values = self.get_form_values(response)
+        values['body'] = 'test subscribtion юникод'
+        response = self.client.post(add_post_url, values, follow=True)
+        self.assertEqual(response.status_code, 200)
+        new_post = Post.objects.get(pk=2)
+
+        # there should only be one email in the outbox (to user2@example.com)
+        self.assertEqual(len(mail.outbox),1)
+        self.assertTrue([msg for msg in mail.outbox if new_post.get_absolute_url() in msg.body])
+
+        # unsubscribe
+        client.login(username='user2', password='user2')
         self.assertTrue([msg for msg in mail.outbox if new_post.get_absolute_url() in msg.body])
         response = client.get(reverse('pybb:delete_subscription', args=[self.topic.id]), follow=True)
         self.assertEqual(response.status_code, 200)
@@ -552,16 +764,51 @@ class FeaturesTest(TransactionTestCase, SharedTestModule):
         response = client.get(self.forum.get_absolute_url())
         self.assertEqual(response.context['topic_list'][0], self.topic)
 
+    def test_topic_deleted(self):
+        forum_1 = Forum.objects.create(name='new forum', category=self.category)
+        topic_1 = Topic.objects.create(name='new topic', forum=forum_1, user=self.user)
+        post_1 = Post.objects.create(topic=topic_1, user=self.user, body='test')
+        time.sleep(2)
+        self.assertEqual(topic_1.updated, post_1.created)
+        self.assertEqual(forum_1.updated, post_1.created)
+
+        topic_2 = Topic.objects.create(name='another topic', forum=forum_1, user=self.user)
+        post_2 = Post.objects.create(topic=topic_2, user=self.user, body='another test')
+        time.sleep(2)
+        self.assertEqual(topic_2.updated, post_2.created)
+        self.assertEqual(forum_1.updated, post_2.created)
+
+        topic_2.delete()
+        self.assertEqual(forum_1.updated, post_1.created)
+        self.assertEqual(forum_1.topic_count, 1)
+        self.assertEqual(forum_1.post_count, 1)
+
+        post_1.delete()
+        self.assertEqual(forum_1.topic_count, 0)
+        self.assertEqual(forum_1.post_count, 0)
+
     def test_user_view(self):
         resp = self.client.get(reverse('pybb:user', kwargs={'username': self.user.username}))
         self.assertEqual(resp.status_code, 200)
 
+    def test_post_count(self):
+        topic = Topic(name='etopic', forum=self.forum, user=self.user)
+        topic.save()
+        post = Post(topic=topic, user=self.user, body='test') # another post
+        post.save()
+        self.assertEqual(self.user.get_profile().post_count, 2)
+        post.body = 'test2'
+        post.save()
+        self.assertEqual(Profile.objects.get(pk=self.user.get_profile().pk).post_count, 2)
+        post.delete()
+        self.assertEqual(Profile.objects.get(pk=self.user.get_profile().pk).post_count, 1)
+
     def tearDown(self):
         defaults.PYBB_ENABLE_ANONYMOUS_POST = self.ORIG_PYBB_ENABLE_ANONYMOUS_POST
         defaults.PYBB_PREMODERATION = self.ORIG_PYBB_PREMODERATION
 
 
-class AnonymousTest(TransactionTestCase, SharedTestModule):
+class AnonymousTest(TestCase, SharedTestModule):
 
     def setUp(self):
         self.ORIG_PYBB_ENABLE_ANONYMOUS_POST = defaults.PYBB_ENABLE_ANONYMOUS_POST
@@ -599,7 +846,7 @@ def premoderate_test(user, post):
         return True
     return False
 
-class PreModerationTest(TransactionTestCase, SharedTestModule):
+class PreModerationTest(TestCase, SharedTestModule):
 
     def setUp(self):
         self.ORIG_PYBB_PREMODERATION = defaults.PYBB_PREMODERATION
@@ -624,10 +871,10 @@ class PreModerationTest(TransactionTestCase, SharedTestModule):
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, 'test premoderation')
 
-        # Post is not visible by others
+        # Post is not visible by anonymous user
         client = Client()
         response = client.get(post.get_absolute_url(), follow=True)
-        self.assertEqual(response.status_code, 403)
+        self.assertRedirects(response, settings.LOGIN_URL+'?next=%s' % post.get_absolute_url())
         response = client.get(self.topic.get_absolute_url(), follow=True)
         self.assertNotContains(response, 'test premoderation')
 
@@ -693,7 +940,7 @@ class PreModerationTest(TransactionTestCase, SharedTestModule):
         self.assertEqual(response.status_code, 200)
         self.assertNotContains(response, 'new topic name')
         response = client.get(Topic.objects.get(name='new topic name').get_absolute_url())
-        self.assertEqual(response.status_code, 403)
+        self.assertEqual(response.status_code, 302)
         response = admin_client.get(reverse('pybb:moderate_post',
                                      kwargs={'pk': Post.objects.get(body='new topic test').id}),
                                      follow=True)
@@ -709,7 +956,7 @@ class PreModerationTest(TransactionTestCase, SharedTestModule):
         defaults.PYBB_PREMODERATION = self.ORIG_PYBB_PREMODERATION
 
         
-class AttachmentTest(TransactionTestCase, SharedTestModule):
+class AttachmentTest(TestCase, SharedTestModule):
     def setUp(self):
         self.PYBB_ATTACHMENT_ENABLE = defaults.PYBB_ATTACHMENT_ENABLE
         defaults.PYBB_ATTACHMENT_ENABLE = True
@@ -719,7 +966,7 @@ class AttachmentTest(TransactionTestCase, SharedTestModule):
         self.create_user()
         self.create_initial()
 
-    def test_attachment(self):
+    def test_attachment_one(self):
         add_post_url = reverse('pybb:add_post', kwargs={'topic_id': self.topic.id})
         self.login_client()
         response = self.client.get(add_post_url)
@@ -730,7 +977,7 @@ class AttachmentTest(TransactionTestCase, SharedTestModule):
         self.assertEqual(response.status_code, 200)
         self.assertTrue(Post.objects.filter(body='test attachment').exists())
 
-    def test_attachment(self):
+    def test_attachment_two(self):
         add_post_url = reverse('pybb:add_post', kwargs={'topic_id': self.topic.id})
         self.login_client()
         response = self.client.get(add_post_url)
@@ -747,7 +994,7 @@ class AttachmentTest(TransactionTestCase, SharedTestModule):
         defaults.PYBB_PREMODERATION = self.ORIG_PYBB_PREMODERATION
 
 
-class PollTest(TransactionTestCase, SharedTestModule):
+class PollTest(TestCase, SharedTestModule):
     def setUp(self):
         self.create_user()
         self.create_initial()
@@ -800,6 +1047,23 @@ class PollTest(TransactionTestCase, SharedTestModule):
         self.assertEqual(new_topic.poll_question, 'q1')
         self.assertEqual(PollAnswer.objects.filter(topic=new_topic).count(), 2)
 
+    def test_regression_adding_poll_with_removed_answers(self):
+        add_topic_url = reverse('pybb:add_topic', kwargs={'forum_id': self.forum.id})
+        self.login_client()
+        response = self.client.get(add_topic_url)
+        values = self.get_form_values(response)
+        values['body'] = 'test poll body'
+        values['name'] = 'test poll name'
+        values['poll_type'] = 1
+        values['poll_question'] = 'q1'
+        values['poll_answers-0-text'] = ''
+        values['poll_answers-0-DELETE'] = 'on'
+        values['poll_answers-1-text'] = ''
+        values['poll_answers-1-DELETE'] = 'on'
+        values['poll_answers-TOTAL_FORMS'] = 2
+        response = self.client.post(add_topic_url, values, follow=True)
+        self.assertFalse(Topic.objects.filter(name='test poll name').exists())
+
     def test_regression_poll_deletion_after_second_post(self):
         self.login_client()
 
@@ -901,7 +1165,7 @@ class PollTest(TransactionTestCase, SharedTestModule):
         defaults.PYBB_POLL_MAX_ANSWERS = self.PYBB_POLL_MAX_ANSWERS
 
 
-class FiltersTest(TransactionTestCase, SharedTestModule):
+class FiltersTest(TestCase, SharedTestModule):
     def setUp(self):
         self.create_user()
         self.create_initial(post=False)
@@ -914,4 +1178,172 @@ class FiltersTest(TransactionTestCase, SharedTestModule):
         values['body'] = u'test\n \n \n\nmultiple empty lines\n'
         response = self.client.post(add_post_url, values, follow=True)
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(Post.objects.all()[0].body, u'test\nmultiple empty lines')
\ No newline at end of file
+        self.assertEqual(Post.objects.all()[0].body, u'test\nmultiple empty lines')
+        
+from pybb import permissions
+from django.db.models import Q
+
+class CustomPermissionHandler(permissions.DefaultPermissionHandler):
+    """ 
+    a custom permission handler which changes the meaning of "hidden" forum:
+    "hidden" forum or category is visible for all logged on users, not only staff 
+    """
+        
+    def filter_categories(self, user, qs):
+        return qs.filter(hidden=False) if user.is_anonymous() else qs
+    
+    def may_view_category(self, user, category):
+        return user.is_authenticated() if category.hidden else True
+    
+    def filter_forums(self, user, qs):
+        if user.is_anonymous():
+            qs = qs.filter(Q(hidden=False)&Q(category__hidden=False))
+        return qs        
+    
+    def may_view_forum(self, user, forum):
+        return user.is_authenticated() if forum.hidden or forum.category.hidden else True
+    
+    def filter_topics(self, user, qs):
+        if user.is_anonymous():
+            qs = qs.filter(Q(forum__hidden=False)&Q(forum__category__hidden=False))
+        return qs
+
+    def may_view_topic(self, user, topic):
+        return self.may_view_forum(user, topic.forum)
+
+    def filter_posts(self, user, qs):
+        if user.is_anonymous():
+            qs = qs.filter(Q(topic__forum__hidden=False)&Q(topic__forum__category__hidden=False))
+        return qs
+
+    def may_view_post(self, user, post):
+        return self.may_view_forum(user, post.topic.forum)
+
+
+class CustomPermissionHandlerTest(TestCase, SharedTestModule):
+    """ test custom permission handler """
+    
+    def setUp(self):
+        from pybb import views
+        self.create_user()
+        # create public and hidden categories, forums, posts
+        c_pub = Category(name='public'); c_pub.save()
+        c_hid = Category(name='private', hidden=True); c_hid.save()
+        Forum(name='pub1', category=c_pub).save()
+        Forum(name='priv1', category=c_hid).save()
+        Forum(name='private_in_public_cat', hidden=True, category=c_pub).save()
+        for f in Forum.objects.all():
+            t = Topic(name='a topic', forum=f, user=self.user)
+            t.save()
+            Post(topic=t, user=self.user, body='test').save()
+        
+        # override the permission handler. this cannot be done with @override_settings as
+        # permissions.perms is already imported at this point, instead we got to monkeypatch
+        # the modules (not really nice, but only an issue in tests)
+        views.perms = permissions.perms = permissions._resolve_class('pybb.tests.CustomPermissionHandler')
+    
+    def tearDown(self):
+        from pybb import views
+        # reset permission handler (otherwise other tests may fail)
+        views.perms = permissions.perms = permissions._resolve_class('pybb.permissions.DefaultPermissionHandler')
+    
+    def test_category_permission(self):
+        for c in Category.objects.all():
+            # anon user may not see category
+            r=self.get_with_user(c.get_absolute_url())
+            if c.hidden:
+                self.assertEqual(r.status_code, 302)
+            else:
+                self.assertEqual(r.status_code, 200)
+            # logged on user may see all categories
+            r=self.get_with_user(c.get_absolute_url(), 'zeus', 'zeus')        
+            self.assertEqual(r.status_code, 200)
+            
+    def test_forum_permission(self):
+        for f in Forum.objects.all():
+            r=self.get_with_user(f.get_absolute_url())
+            self.assertEqual(r.status_code, 302 if f.hidden or f.category.hidden else 200)
+            r=self.get_with_user(f.get_absolute_url(), 'zeus', 'zeus')            
+            self.assertEqual(r.status_code, 200)
+            
+    def test_topic_permission(self):
+        for t in Topic.objects.all():
+            r=self.get_with_user(t.get_absolute_url())
+            self.assertEqual(r.status_code, 302 if t.forum.hidden or t.forum.category.hidden else 200)
+            r=self.get_with_user(t.get_absolute_url(), 'zeus', 'zeus')            
+            self.assertEqual(r.status_code, 200)
+            
+    def test_post_permission(self):
+        for p in Post.objects.all():
+            r=self.get_with_user(p.get_absolute_url())
+            self.assertEqual(r.status_code, 302 if p.topic.forum.hidden or p.topic.forum.category.hidden else 301)
+            r=self.get_with_user(p.get_absolute_url(), 'zeus', 'zeus')            
+            self.assertEqual(r.status_code, 301)
+            
+class LogonRedirectTest(TestCase, SharedTestModule):
+    """ test whether anonymous user gets redirected, whereas unauthorized user gets PermissionDenied """
+    
+    def setUp(self):
+        # create users
+        staff = User.objects.create_user('staff', 'staff@localhost', 'staff')
+        staff.is_staff = True
+        staff.save()
+        nostaff = User.objects.create_user('nostaff', 'nostaff@localhost', 'nostaff')
+        nostaff.is_staff = False
+        nostaff.save()
+        
+        # create topic, post in hidden category 
+        self.category = Category(name='private', hidden=True)
+        self.category.save()
+        self.forum = Forum(name='priv1', category=self.category)
+        self.forum.save()
+        self.topic = Topic(name='a topic', forum=self.forum, user=staff)
+        self.topic.save()
+        self.post = Post(body='body post', topic=self.topic, user=staff, on_moderation=True)
+        self.post.save()            
+                    
+    def test_redirect_category(self):
+        # access without user should be redirected
+        r = self.get_with_user(self.category.get_absolute_url())
+        self.assertRedirects(r, settings.LOGIN_URL+'?next=%s' % self.category.get_absolute_url())
+        # access with (unauthorized) user should get 403 (forbidden)
+        r = self.get_with_user(self.category.get_absolute_url(), 'nostaff', 'nostaff')
+        self.assertEquals(r.status_code, 403)
+        # allowed user is allowed
+        r = self.get_with_user(self.category.get_absolute_url(), 'staff', 'staff')
+        self.assertEquals(r.status_code, 200)
+    
+    def test_redirect_forum(self):
+        # access without user should be redirected
+        r = self.get_with_user(self.forum.get_absolute_url())
+        self.assertRedirects(r, settings.LOGIN_URL+'?next=%s' % self.forum.get_absolute_url())
+        # access with (unauthorized) user should get 403 (forbidden)
+        r = self.get_with_user(self.forum.get_absolute_url(), 'nostaff', 'nostaff')
+        self.assertEquals(r.status_code, 403)
+        # allowed user is allowed
+        r = self.get_with_user(self.forum.get_absolute_url(), 'staff', 'staff')
+        self.assertEquals(r.status_code, 200)
+    
+    def test_redirect_topic(self):
+        # access without user should be redirected
+        r = self.get_with_user(self.topic.get_absolute_url())
+        self.assertRedirects(r, settings.LOGIN_URL+'?next=%s' % self.topic.get_absolute_url())
+        # access with (unauthorized) user should get 403 (forbidden)
+        r = self.get_with_user(self.topic.get_absolute_url(), 'nostaff', 'nostaff')
+        self.assertEquals(r.status_code, 403)
+        # allowed user is allowed
+        r = self.get_with_user(self.topic.get_absolute_url(), 'staff', 'staff')
+        self.assertEquals(r.status_code, 200)
+    
+    def test_redirect_post(self):
+        # access without user should be redirected
+        r = self.get_with_user(self.post.get_absolute_url())
+        self.assertRedirects(r, settings.LOGIN_URL+'?next=%s' % self.post.get_absolute_url())
+        # access with (unauthorized) user should get 403 (forbidden)
+        r = self.get_with_user(self.post.get_absolute_url(), 'nostaff', 'nostaff')
+        self.assertEquals(r.status_code, 403)
+        # allowed user is allowed
+        r = self.get_with_user(self.post.get_absolute_url(), 'staff', 'staff')
+        self.assertEquals(r.status_code, 301)
+
+    
\ No newline at end of file
