@@ -3,14 +3,14 @@
 import math
 
 from django.contrib.auth.models import User
-from django.contrib.auth.decorators import login_required, permission_required
+from django.contrib.auth.decorators import login_required
 from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
 from django.core.urlresolvers import reverse
 from django.contrib import messages
-from django.db import transaction
 from django.db.models import F, Q
-from django.http import HttpResponseRedirect, HttpResponse, Http404, HttpResponseBadRequest
-from django.shortcuts import get_object_or_404, redirect, _get_queryset
+from django.http import HttpResponseRedirect, HttpResponse, Http404, HttpResponseBadRequest,\
+    HttpResponseForbidden
+from django.shortcuts import get_object_or_404, redirect, render
 from django.utils.translation import ugettext_lazy as _
 from django.utils.decorators import method_decorator
 from django.views.generic.edit import ModelFormMixin
@@ -28,19 +28,29 @@ from pure_pagination import Paginator
 
 from pybb.models import Category, Forum, Topic, Post, TopicReadTracker, ForumReadTracker, PollAnswerUser
 from pybb.forms import  PostForm, AdminPostForm, EditProfileForm, AttachmentFormSet, PollAnswerFormSet, PollForm
-from pybb.templatetags.pybb_tags import pybb_editable_by, pybb_topic_poll_not_voted
-from pybb.templatetags.pybb_tags import pybb_topic_moderated_by
+from pybb.templatetags.pybb_tags import pybb_topic_poll_not_voted
 from pybb import defaults
 
+from pybb.permissions import perms
 
-def filter_hidden(request, queryset_or_model):
-    """
-    Return queryset for model, manager or queryset, filtering hidden objects for non staff users.
-    """
-    queryset = _get_queryset(queryset_or_model)
-    if request.user.is_staff:
-        return queryset
-    return queryset.filter(hidden=False)
+class RedirectToLoginMixin(object):
+    """ mixin which redirects to settings.LOGIN_URL if the view encounters an PermissionDenied exception
+        and the user is not authenticated. Views inheriting from this need to implement 
+        get_login_redirect_url(), which returns the URL to redirect to after login (parameter "next")
+    """ 
+    def dispatch(self, *args, **kwargs):
+        try:
+            return super(RedirectToLoginMixin, self).dispatch(*args, **kwargs)
+        except PermissionDenied:
+            if not self.request.user.is_authenticated():
+                from django.contrib.auth.views import redirect_to_login
+                return redirect_to_login(self.get_login_redirect_url())
+            else:
+                return HttpResponseForbidden()
+
+    def get_login_redirect_url(self):
+        """ get the url to which we redirect after the user logs in. subclasses should override this """
+        return '/'
 
 class IndexView(generic.ListView):
 
@@ -49,47 +59,58 @@ class IndexView(generic.ListView):
 
     def get_context_data(self, **kwargs):
         ctx = super(IndexView, self).get_context_data(**kwargs)
-        categories = list(ctx['categories'])
+        categories = ctx['categories']
         for category in categories:
-            category.forums_accessed = filter_hidden(self.request, category.forums.all())
+            category.forums_accessed = perms.filter_forums(self.request.user, category.forums.all())
         ctx['categories'] = categories
         return ctx
 
     def get_queryset(self):
-        return filter_hidden(self.request, Category)
-
+        return perms.filter_categories(self.request.user, Category.objects.all())
     
-class CategoryView(generic.DetailView):
+class CategoryView(RedirectToLoginMixin, generic.DetailView):
 
     template_name = 'pybb/index.html'
     context_object_name = 'category'
 
+    def get_login_redirect_url(self):
+        return reverse('pybb:category', args=(self.kwargs['pk'],))
+    
     def get_queryset(self):
-        return filter_hidden(self.request, Category)
+        return Category.objects.all()
+    
+    def get_object(self, queryset=None):
+        obj = super(CategoryView, self).get_object(queryset)
+        if not perms.may_view_category(self.request.user, obj):
+            raise PermissionDenied
+        return obj        
 
     def get_context_data(self, **kwargs):
         ctx = super(CategoryView, self).get_context_data(**kwargs)
-        ctx['category'].forums_accessed = filter_hidden(self.request, ctx['category'].forums.all())
+        ctx['category'].forums_accessed = perms.filter_forums(self.request.user, ctx['category'].forums.all())
         ctx['categories'] = [ctx['category']]
         return ctx
 
-
-class ForumView(generic.ListView):
+class ForumView(RedirectToLoginMixin, generic.ListView):
 
     paginate_by = defaults.PYBB_FORUM_PAGE_SIZE
     context_object_name = 'topic_list'
     template_name = 'pybb/forum.html'
     paginator_class = Paginator
 
+    def get_login_redirect_url(self):
+        return reverse('pybb:forum', args=(self.kwargs['pk'],))
+        
     def get_context_data(self, **kwargs):
         ctx = super(ForumView, self).get_context_data(**kwargs)
         ctx['forum'] = self.forum
         return ctx
 
     def get_queryset(self):
-        self.forum = get_object_or_404(filter_hidden(self.request, Forum), pk=self.kwargs['pk'])
-        if self.forum.category.hidden and (not self.request.user.is_staff):
-            raise Http404()
+        self.forum = get_object_or_404(Forum.objects.all(), pk=self.kwargs['pk'])
+        if not perms.may_view_forum(self.request.user, self.forum):
+            raise PermissionDenied
+        
         qs = self.forum.topics.order_by('-sticky', '-updated').select_related()
         if not (self.request.user.is_superuser or self.request.user in self.forum.moderators.all()):
             if self.request.user.is_authenticated():
@@ -99,28 +120,38 @@ class ForumView(generic.ListView):
         return qs
     
 
-class TopicView(generic.ListView):
+class LatestTopicsView(generic.ListView):
+
+    paginate_by = defaults.PYBB_FORUM_PAGE_SIZE
+    context_object_name = 'topic_list'
+    template_name = 'pybb/latest_topics.html'
+    paginator_class = Paginator
+
+    def get_queryset(self):
+        qs = Topic.objects.all().select_related()
+        qs = perms.filter_topics(self.request.user, qs)
+        return qs.order_by('-updated')
+
+
+class TopicView(RedirectToLoginMixin, generic.ListView):
     paginate_by = defaults.PYBB_TOPIC_PAGE_SIZE
     template_object_name = 'post_list'
     template_name = 'pybb/topic.html'
     paginator_class = Paginator
 
-    def get_queryset(self):
+    def get_login_redirect_url(self):
+        return reverse('pybb:topic', args=(self.kwargs['pk'],))    
+
+    def get_queryset(self):        
         self.topic = get_object_or_404(Topic.objects.select_related('forum'), pk=self.kwargs['pk'])
-        if self.topic.on_moderation and\
-           not pybb_topic_moderated_by(self.topic, self.request.user) and\
-           not self.request.user == self.topic.user:
+        if not perms.may_view_topic(self.request.user, self.topic):
             raise PermissionDenied
-        if (self.topic.forum.hidden or self.topic.forum.category.hidden) and (not self.request.user.is_staff):
-            raise Http404()
+            
         self.topic.views += 1
         self.topic.save()
         qs = self.topic.posts.all().select_related('user')
-        if not pybb_topic_moderated_by(self.topic, self.request.user):
-            if self.request.user.is_authenticated():
-                qs = qs.filter(Q(user=self.request.user)|Q(on_moderation=False))
-            else:
-                qs = qs.filter(on_moderation=False)
+        if not perms.may_moderate_topic(self.request.user, self.topic):
+            qs = perms.filter_posts(self.request.user, qs)
         return qs
 
     def get_context_data(self, **kwargs):
@@ -128,7 +159,7 @@ class TopicView(generic.ListView):
         if self.request.user.is_authenticated():
             self.request.user.is_moderator = self.request.user.is_superuser or (self.request.user in self.topic.forum.moderators.all())
             self.request.user.is_subscribed = self.request.user in self.topic.subscribers.all()
-            if self.request.user.is_staff:
+            if perms.may_post_as_admin(self.request.user):
                 ctx['form'] = AdminPostForm(initial={'login': self.request.user.username}, topic=self.topic)
             else:
                 ctx['form'] = PostForm(topic=self.topic)
@@ -159,7 +190,7 @@ class TopicView(generic.ListView):
             forum_mark = None
         if (forum_mark is None) or (forum_mark.time_stamp < topic.updated):
             # Mark topic as readed
-            topic_mark, new = TopicReadTracker.objects.get_or_create(topic=topic, user=request.user)
+            topic_mark, new = TopicReadTracker.objects.get_or_create_tracker(topic=topic, user=request.user)
             if not new:
                 topic_mark.save()
 
@@ -173,14 +204,14 @@ class TopicView(generic.ListView):
                         user=request.user,
                         topic__forum=topic.forum
                         ).delete()
-                forum_mark, new = ForumReadTracker.objects.get_or_create(forum=topic.forum, user=request.user)
+                forum_mark, new = ForumReadTracker.objects.get_or_create_tracker(forum=topic.forum, user=request.user)
                 forum_mark.save()
 
 
 class PostEditMixin(object):
 
-    def get_form_class(self):
-        if self.request.user.is_staff:
+    def get_form_class(self):        
+        if perms.may_post_as_admin(self.request.user):
             return AdminPostForm
         else:
             return PostForm
@@ -195,41 +226,42 @@ class PostEditMixin(object):
 
     def form_valid(self, form):
         success = True
-        with transaction.commit_manually():
-            try:
-                self.object = form.save()
+        save_attachments = False
+        save_poll_answers = False
+        self.object = form.save(commit=False)
 
-                if defaults.PYBB_ATTACHMENT_ENABLE:
-                    aformset = AttachmentFormSet(self.request.POST, self.request.FILES, instance=self.object)
-                    if aformset.is_valid():
-                        aformset.save()
-                    else:
-                        success = False
+        if defaults.PYBB_ATTACHMENT_ENABLE:
+            aformset = AttachmentFormSet(self.request.POST, self.request.FILES, instance=self.object)
+            if aformset.is_valid():
+                save_attachments = True
+            else:
+                success = False
+        else:
+            aformset = AttachmentFormSet()
+
+        pollformset = PollAnswerFormSet()
+        if getattr(self, 'forum', None) or self.object.topic.head == self.object:
+            if self.object.topic.poll_type != Topic.POLL_TYPE_NONE:
+                pollformset = PollAnswerFormSet(self.request.POST, instance=self.object.topic)
+                if pollformset.is_valid():
+                    save_poll_answers = True
                 else:
-                    aformset = AttachmentFormSet()
+                    success = False
+            else:
+                self.object.topic.poll_question = None
+                self.object.topic.poll_answers.all().delete()
 
-                if self.object.topic.head == self.object:
-                    if self.object.topic.poll_type != Topic.POLL_TYPE_NONE:
-                        pollformset = PollAnswerFormSet(self.request.POST, instance=self.object.topic)
-                        if pollformset.is_valid():
-                            pollformset.save()
-                        else:
-                            success = False
-                    else:
-                        self.object.topic.poll_question = None
-                        self.object.topic.save()
-                        self.object.topic.poll_answers.all().delete()
-                        pollformset = PollAnswerFormSet()
-
-                if success:
-                    transaction.commit()
-                    return super(ModelFormMixin, self).form_valid(form)
-                else:
-                    transaction.rollback()
-                    return self.render_to_response(self.get_context_data(form=form, aformset=aformset, pollformset=pollformset))
-            except Exception:
-                transaction.rollback()
-                raise
+        if success:
+            self.object.topic.save()
+            self.object.topic = self.object.topic
+            self.object.save()
+            if save_attachments:
+                aformset.save()
+            if save_poll_answers:
+                pollformset.save()
+            return super(ModelFormMixin, self).form_valid(form)
+        else:
+            return self.render_to_response(self.get_context_data(form=form, aformset=aformset, pollformset=pollformset))
 
 
 class AddPostView(PostEditMixin, generic.CreateView):
@@ -275,23 +307,19 @@ class AddPostView(PostEditMixin, generic.CreateView):
             else:
                 from django.contrib.auth.views import redirect_to_login
                 return redirect_to_login(request.get_full_path())
-
-        if not self.user.has_perm('pybb.add_post'):
-            raise PermissionDenied
-
+        
         self.forum = None
         self.topic = None
         if 'forum_id' in kwargs:
-            self.forum = get_object_or_404(filter_hidden(request, Forum), pk=kwargs['forum_id'])
+            self.forum = get_object_or_404(perms.filter_forums(request.user, Forum.objects.all()), pk=kwargs['forum_id'])
+            if not perms.may_create_topic(self.user, self.forum):
+                raise PermissionDenied    
         elif 'topic_id' in kwargs:
-            self.topic = get_object_or_404(Topic, pk=kwargs['topic_id'])
-            if self.topic.forum.hidden and (not self.user.is_staff):
-                raise Http404
-            if self.topic.closed:
-                raise PermissionDenied
+            self.topic = get_object_or_404(perms.filter_topics(request.user, Topic.objects.all()), pk=kwargs['topic_id'])
+            if not perms.may_create_post(self.user, self.topic):
+                raise PermissionDenied            
         return super(AddPostView, self).dispatch(request, *args, **kwargs)
 
-
 class EditPostView(PostEditMixin, generic.UpdateView):
 
     model = Post
@@ -301,12 +329,12 @@ class EditPostView(PostEditMixin, generic.UpdateView):
 
     @method_decorator(login_required)
     @method_decorator(csrf_protect)
-    def dispatch(self, request, *args, **kwargs):
+    def dispatch(self, request, *args, **kwargs):        
         return super(EditPostView, self).dispatch(request, *args, **kwargs)
 
     def get_object(self, queryset=None):
         post = super(EditPostView, self).get_object(queryset)
-        if not pybb_editable_by(post, self.request.user):
+        if not perms.may_edit_post(self.request.user, post):
             raise PermissionDenied
         return post
 
@@ -327,13 +355,14 @@ class UserView(generic.DetailView):
         return ctx
         
 
-class PostView(generic.RedirectView):
+class PostView(RedirectToLoginMixin, generic.RedirectView):
+    
+    def get_login_redirect_url(self):
+        return reverse('pybb:post', args=(self.kwargs['pk'],))
+    
     def get_redirect_url(self, **kwargs):
-        post = get_object_or_404(Post, pk=self.kwargs['pk'])
-        if defaults.PYBB_PREMODERATION and\
-            post.on_moderation and\
-            (not pybb_topic_moderated_by(post.topic, self.request.user)) and\
-            (not post.user==self.request.user):
+        post = get_object_or_404(Post.objects.all(), pk=self.kwargs['pk'])
+        if not perms.may_view_post(self.request.user, post):
             raise PermissionDenied
         count = post.topic.posts.filter(created__lt=post.created).count() + 1
         page = math.ceil(count / float(defaults.PYBB_TOPIC_PAGE_SIZE))
@@ -343,7 +372,7 @@ class PostView(generic.RedirectView):
 class ModeratePost(generic.RedirectView):
     def get_redirect_url(self, **kwargs):
         post = get_object_or_404(Post, pk=self.kwargs['pk'])
-        if not pybb_topic_moderated_by(post.topic, self.request.user):
+        if not perms.may_moderate_topic(self.request.user, post.topic):
             raise PermissionDenied
         post.on_moderation = False
         post.save()
@@ -374,58 +403,81 @@ class DeletePostView(generic.DeleteView):
 
     def get_object(self, queryset=None):
         post = get_object_or_404(Post.objects.select_related('topic', 'topic__forum'), pk=self.kwargs['pk'])
+        if not perms.may_delete_post(self.request.user, post):
+            raise PermissionDenied
         self.topic = post.topic
         self.forum = post.topic.forum
-        if not pybb_topic_moderated_by(self.topic, self.request.user):
-           raise PermissionDenied
+        if not perms.may_moderate_topic(self.request.user, self.topic):
+            raise PermissionDenied
         return post
 
+    def delete(self, request, *args, **kwargs):
+        self.object = self.get_object()
+        self.object.delete()
+        redirect_url = self.get_success_url()
+        if not request.is_ajax():
+            return HttpResponseRedirect(redirect_url)
+        else:
+            return HttpResponse(redirect_url)
+
     def get_success_url(self):
         try:
             Topic.objects.get(pk=self.topic.id)
         except Topic.DoesNotExist:
             return self.forum.get_absolute_url()
         else:
-            return self.topic.get_absolute_url()
+            if not self.request.is_ajax():
+                return self.topic.get_absolute_url()
+            else:
+                return ""
 
 
 class TopicActionBaseView(generic.View):
 
     def get_topic(self):
-        topic = get_object_or_404(Topic, pk=self.kwargs['pk'])
-        if not pybb_topic_moderated_by(topic, self.request.user):
-            raise PermissionDenied
-        return topic
-
+        return get_object_or_404(Topic, pk=self.kwargs['pk'])
+        
     @method_decorator(login_required)
-    def get(self, *args, **kwargs):
+    def get(self, *args, **kwargs):        
         self.topic = self.get_topic()
         self.action(self.topic)
         return HttpResponseRedirect(self.topic.get_absolute_url())
 
-
 class StickTopicView(TopicActionBaseView):
+        
     def action(self, topic):
+        if not perms.may_stick_topic(self.request.user, topic):
+            raise PermissionDenied
         topic.sticky = True
         topic.save()
 
 
 class UnstickTopicView(TopicActionBaseView):
+    
     def action(self, topic):
+        if not perms.may_unstick_topic(self.request.user, topic):
+            raise PermissionDenied        
         topic.sticky = False
         topic.save()
 
 
 class CloseTopicView(TopicActionBaseView):
+    
     def action(self, topic):
+        if not perms.may_close_topic(self.request.user, topic):
+            raise PermissionDenied        
         topic.closed = True
         topic.save()
 
+
 class OpenTopicView(TopicActionBaseView):
     def action(self, topic):
+        if not perms.may_open_topic(self.request.user, topic):
+            raise PermissionDenied        
         topic.closed = False
         topic.save()
 
+
 class TopicPollVoteView(generic.UpdateView):
     model = Topic
     http_method_names = ['post', ]
@@ -463,38 +515,43 @@ class TopicPollVoteView(generic.UpdateView):
 
 @login_required
 def delete_subscription(request, topic_id):
-    topic = get_object_or_404(Topic, pk=topic_id)
+    topic = get_object_or_404(perms.filter_topics(request.user, Topic.objects.all()), pk=topic_id)
     topic.subscribers.remove(request.user)
     return HttpResponseRedirect(topic.get_absolute_url())
 
+
 @login_required
 def add_subscription(request, topic_id):
-    topic = get_object_or_404(Topic, pk=topic_id)
+    topic = get_object_or_404(perms.filter_topics(request.user, Topic.objects.all()), pk=topic_id)
     topic.subscribers.add(request.user)
     return HttpResponseRedirect(topic.get_absolute_url())
 
+
 @login_required
 def post_ajax_preview(request):
     content = request.POST.get('data')
     html = defaults.PYBB_MARKUP_ENGINES[defaults.PYBB_MARKUP](content)
-    return HttpResponse(html)
+    return render(request, 'pybb/_markitup_preview.html', {'html': html})
+
 
 @login_required
 def mark_all_as_read(request):
-    for forum in filter_hidden(request, Forum):
-        forum_mark, new = ForumReadTracker.objects.get_or_create(forum=forum, user=request.user)
+    for forum in perms.filter_forums(request.user, Forum.objects.all()):
+        forum_mark, new = ForumReadTracker.objects.get_or_create_tracker(forum=forum, user=request.user)
         forum_mark.save()
     TopicReadTracker.objects.filter(user=request.user).delete()
     msg = _('All forums marked as read')
     messages.success(request, msg, fail_silently=True)
     return redirect(reverse('pybb:index'))
 
+
 @login_required
-@permission_required('pybb.block_users')
 def block_user(request, username):
     user = get_object_or_404(User, username=username)
+    if not perms.may_block_user(request.user, user):
+        raise PermissionDenied
     user.is_active = False
     user.save()
     msg = _('User successfuly blocked')
     messages.success(request, msg, fail_silently=True)
-    return redirect('pybb:index')
\ No newline at end of file
+    return redirect('pybb:index')
