@@ -12,7 +12,7 @@ TEMPLATE_DEBUG = DEBUG
 DATABASES = {
     'default': {
         'ENGINE': 'django.db.backends.sqlite3', # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
-        'NAME': '../testdb.sqlite',                      # Or path to database file if using sqlite3.
+        'NAME': 'testdb.sqlite',                      # Or path to database file if using sqlite3.
     }
 }
 
@@ -56,7 +56,6 @@ MIDDLEWARE_CLASSES = (
     'django.middleware.clickjacking.XFrameOptionsMiddleware',
     'account.middleware.LocaleMiddleware',
     'pybb.middleware.PybbMiddleware',
-    #'debug_toolbar.middleware.DebugToolbarMiddleware',
 )
 
 ROOT_URLCONF = 'example_thirdparty.urls'
@@ -84,11 +83,8 @@ INSTALLED_APPS = (
     'pinax_theme_bootstrap_account',
     'pinax_theme_bootstrap',
     'django_forms_bootstrap',
-    'debug_toolbar',
 )
 
-INTERNAL_IPS = ('127.0.0.1', )
-
 TEMPLATE_CONTEXT_PROCESSORS = [
     'django.contrib.auth.context_processors.auth',
     'django.core.context_processors.debug',
@@ -102,37 +98,8 @@ TEMPLATE_CONTEXT_PROCESSORS = [
 ]
 AUTH_PROFILE_MODULE = 'pybb.Profile'
 
-DEBUG_TOOLBAR_CONFIG = {
-    'INTERCEPT_REDIRECTS': False
-}
-
-LOGGING = {
-    'version': 1,
-    'disable_existing_loggers': False,
-    'filters': {
-        'require_debug_false': {
-            '()': 'django.utils.log.RequireDebugFalse'
-        }
-    },
-    'handlers': {
-        'mail_admins': {
-            'level': 'ERROR',
-            'filters': ['require_debug_false'],
-            'class': 'django.utils.log.AdminEmailHandler'
-        }
-    },
-    'loggers': {
-        'django.request': {
-            'handlers': ['mail_admins'],
-            'level': 'ERROR',
-            'propagate': True,
-        },
-    }
-}
-
 EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
 
 # Pybb
 PYBB_TEMPLATE = "site_base.html"
-PYBB_ATTACHMENT_ENABLE = True
-
+PYBB_ATTACHMENT_ENABLE = True
\ No newline at end of file
